@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/chts.c,v 1.13 2002/06/05 17:09:53 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/chts.c,v 1.27 2004/03/07 01:56:18 hazelsct Exp $
 
   This is the Cahn Hilliard timestepping code.  It is provided here as an
   example usage of the Illuminator Distributed Visualization Library.
@@ -38,8 +38,8 @@ Parallelism can be invoked based on the DA construct:\n\
 
 /* Set #if to 1 to debug, 0 otherwise */
 #undef DPRINTF
-#if 0
-#define DPRINTF(fmt, args...) printf ("%s: " fmt, __FUNCT__)
+#ifdef DEBUG
+#define DPRINTF(fmt, args...) ierr = PetscPrintf (PETSC_COMM_WORLD, "%s: " fmt, __FUNCT__, args); CHKERRQ(ierr)
 #else
 #define DPRINTF(fmt, args...)
 #endif
@@ -155,7 +155,13 @@ int ch_ts_jacobian_3d (TS thets, PetscScalar time, Vec X, Mat *A, Mat *B,
 #define __FUNCT__ "ch_ts_monitor"
 
 /*++++++++++++++++++++++++++++++++++++++
-  Monitor routine which displays the current state.
+  Monitor routine which displays the current state, using
+  +latex+{\tt Illuminator}'s {\tt geomview}
+  +html+ <tt>Illuminator</tt>'s <tt>geomview</tt>
+  front-end (unless -no_contours is used); and also saves it using
+  +latex+{\tt IlluMultiSave()}
+  +html+ <tt>IlluMultiSave()</tt>
+  if -save_data is specified.
 
   int ch_ts_monitor Usual return: zero or error.
 
@@ -191,13 +197,14 @@ int ch_ts_monitor (TS thets, int stepno, PetscScalar time, Vec X, void *ptr)
     {
       if (user->threedee)
 	{
-	  DPRINTF ("Starting triangulation\n");
+	  DPRINTF ("Starting triangulation\n",0);
 	  ierr = DATriangulate (user->da, X, user->chvar, minmax, PETSC_DECIDE,
-				PETSC_NULL, PETSC_NULL); CHKERRQ (ierr);
-	  DPRINTF ("Done, sending to Geomview\n");
+				PETSC_NULL, PETSC_NULL, PETSC_FALSE,
+				PETSC_FALSE, PETSC_FALSE); CHKERRQ (ierr);
+	  DPRINTF ("Done, sending to Geomview\n",0);
 	  ierr = GeomviewDisplayTriangulation
-	    (user->comm, minmax,"Cahn-Hilliard"); CHKERRQ (ierr);
-	  DPRINTF ("Done.\n");
+	    (user->comm, minmax, "Cahn-Hilliard", PETSC_TRUE); CHKERRQ (ierr);
+	  DPRINTF ("Done.\n",0);
 	}
       else
 	{
@@ -205,6 +212,28 @@ int ch_ts_monitor (TS thets, int stepno, PetscScalar time, Vec X, void *ptr)
 	}
     }
 
+  if (user->save_data)
+    {
+      PetscReal deltat;
+      field_plot_type chtype=FIELD_SCALAR_CONTOURS;
+      char filename [100], *paramdata [4],
+	*paramnames [4] = { "time", "timestep", "gamma", "kappa" };
+
+      for (ierr=0; ierr<4; ierr++)
+	paramdata[ierr] = (char *) malloc (50*sizeof(char));
+      snprintf (filename, 99, "chtsout.time%.3d", stepno);
+      TSGetTimeStep (thets, &deltat);
+      snprintf (paramdata[0], 49, "%g", time);
+      snprintf (paramdata[1], 49, "%g", deltat);
+      snprintf (paramdata[2], 49, "%g", user->gamma);
+      snprintf (paramdata[3], 49, "%g", user->kappa);
+      DPRINTF ("Storing data using IlluMultiSave()\n",0);
+      IlluMultiSave (user->da, X, filename, 1.,1.,1., &chtype, 4, paramnames,
+		     paramdata, COMPRESS_INT_NONE | COMPRESS_GZIP_FAST);
+    }
+
+  DPRINTF ("Completed timestep monitor tasks.\n",0);
+
   return 0;
 }
 
@@ -225,7 +254,7 @@ int ch_ts_monitor (TS thets, int stepno, PetscScalar time, Vec X, void *ptr)
 int main (int argc, char **argv)
 {
   TS            thets;               /* the timestepping solver */
-  Vec           x, r;                /* solution, residual vectors */
+  Vec           x;                   /* solution vector */
   AppCtx        user;                /* user-defined work context */
   int           dim, ierr;
   PetscDraw     draw;
@@ -233,7 +262,7 @@ int main (int argc, char **argv)
   PetscReal     xmin, xmax;
   int           temp;
   PetscScalar   ftime, time_total_max = 100.0; /* default max total time */
-  int           steps, time_steps_max = 5;   /* default max timesteps */
+  int           steps = 0, time_steps_max = 5; /* default max timesteps */
 
   PetscInitialize (&argc, &argv, (char *)0, help);
   ierr = MPI_Comm_rank (PETSC_COMM_WORLD, &user.rank); CHKERRQ (ierr);
@@ -244,9 +273,10 @@ int main (int argc, char **argv)
   /* Create user context, set problem data, create vector data structures.
      Also, set the initial condition */
 
-  DPRINTF ("About to initialize problem\n");
+  DPRINTF ("About to initialize problem\n",0);
   ierr = InitializeProblem (&user, &x); CHKERRQ (ierr);
-  ierr = VecDuplicate (x, &r); CHKERRQ (ierr);
+  if (user.load_data > -1)
+    steps = user.load_data;
   ierr = VecDuplicate (user.localX, &user.localF); CHKERRQ (ierr);
 
   /* Set up displays to show graph of the solution */
@@ -269,7 +299,7 @@ int main (int argc, char **argv)
     }
 
   /* Create timestepping solver context */
-  DPRINTF ("Creating timestepping context\n");
+  DPRINTF ("Creating timestepping context\n",0);
   ierr = TSCreate (PETSC_COMM_WORLD, &thets); CHKERRQ (ierr);
   ierr = TSSetProblemType (thets, TS_NONLINEAR); CHKERRQ (ierr);
   ierr = VecGetSize (x, &dim); CHKERRQ (ierr);
@@ -278,7 +308,7 @@ int main (int argc, char **argv)
 		      user.epsilon, user.gamma, user.mparam); CHKERRQ (ierr);
 
   /* Set function evaluation routine and monitor */
-  DPRINTF ("Setting RHS function\n");
+  DPRINTF ("Setting RHS function\n",0);
   if (user.threedee)
     ierr = TSSetRHSFunction (thets, ch_ts_residual_vector_3d, &user);
   else
@@ -293,7 +323,7 @@ int main (int argc, char **argv)
   if (!matfree)
     {
       /* Set up the Jacobian using cahnhill.c subroutines */
-      DPRINTF ("Using analytical Jacobian from cahnhill.c\n");
+      DPRINTF ("Using analytical Jacobian from cahnhill.c\n",0);
       if (user.threedee) {
 	ierr = ch_jacobian_alpha_3d (&user); CHKERRQ (ierr);
 	ierr = TSSetRHSJacobian (thets, user.J, user.J, ch_ts_jacobian_3d,
@@ -317,11 +347,11 @@ int main (int argc, char **argv)
   ierr = TSSetFromOptions (thets); CHKERRQ (ierr);
 
   /* Run the solver */
-  DPRINTF ("Running the solver\n");
+  DPRINTF ("Running the solver\n",0);
   ierr = TSStep (thets, &steps, &ftime); CHKERRQ (ierr);
 
   /* Final clean-up */
-  DPRINTF ("Cleaning up\n");
+  DPRINTF ("Cleaning up\n",0);
   if (!user.no_contours)
     {
       if (user.threedee)
@@ -336,7 +366,6 @@ int main (int argc, char **argv)
   ierr = VecDestroy (user.localX); CHKERRQ (ierr);
   ierr = VecDestroy (x); CHKERRQ (ierr);
   ierr = VecDestroy (user.localF); CHKERRQ (ierr);
-  ierr = VecDestroy (r); CHKERRQ (ierr);      
   ierr = TSDestroy (thets); CHKERRQ (ierr);  
   ierr = PetscFree (user.label); CHKERRQ (ierr);
   ierr = DADestroy (user.da); CHKERRQ (ierr);
@@ -381,16 +410,30 @@ int FormInitialCondition (AppCtx *user, Vec X)
          the array. */
   if (user->random)
     {
-      DPRINTF ("Setting initial condition as random numbers\n");
+      DPRINTF ("Setting initial condition as random numbers\n",0);
       ierr = PetscRandomCreate (user->comm, RANDOM_DEFAULT, &rand);
       CHKERRQ (ierr);
       ierr = PetscRandomSetInterval (rand, 0.35, 0.65); CHKERRQ (ierr);
       ierr = VecSetRandom (rand, X); CHKERRQ (ierr);
       ierr = PetscRandomDestroy (rand); CHKERRQ (ierr);
     }
+  else if (user->load_data > -1)
+    {
+      int usermetacount;
+      char basename [1000], **usermetanames, **usermetadata;
+      sprintf (basename, "chtsout.time%.3d", user->load_data);
+      DPRINTF ("Loading data for timestep %d, basename %s\n", user->load_data,
+	       basename);
+      IlluMultiRead (user->da, X, basename, &usermetacount, &usermetanames,
+		     &usermetadata);
+      /* TODO: free these */
+      for (i=0; i<usermetacount; i++)
+	PetscPrintf (PETSC_COMM_WORLD, "Parameter \"%s\" = \"%s\"\n",
+		     usermetanames [i], usermetadata [i]);
+    }
   else
     {
-      DPRINTF ("Getting local array\n");
+      DPRINTF ("Getting local array\n",0);
       ierr = VecGetArray(localX,&x); CHKERRQ (ierr);
 
       /* Get local grid boundaries (for 2-dimensional DA):
@@ -398,7 +441,7 @@ int FormInitialCondition (AppCtx *user, Vec X)
 	 xm, ym   - widths of local grid (no ghost points)
 	 gxs, gys - starting grid indices (including ghost points)
 	 gxm, gym - widths of local grid (including ghost points) */
-      DPRINTF ("Getting corners and ghost corners\n");
+      DPRINTF ("Getting corners and ghost corners\n",0);
       ierr = DAGetCorners (user->da, &xs,&ys,&zs, &xm,&ym,&zm); CHKERRQ (ierr);
       ierr = DAGetGhostCorners (user->da, &gxs,&gys,&gzs, &gxm,&gym,&gzm);
       CHKERRQ (ierr);
@@ -408,7 +451,7 @@ int FormInitialCondition (AppCtx *user, Vec X)
 
       /* Compute initial condition over the locally owned part of the grid
 	 Initial condition is motionless fluid and equilibrium temperature */
-      DPRINTF ("Looping to set initial condition\n");
+      DPRINTF ("Looping to set initial condition\n",0);
       for (k=zs; k<zs+zm; k++)
 	for (j=ys; j<ys+ym; j++)
 	  for (i=xs; i<xs+xm; i++)
@@ -424,11 +467,11 @@ int FormInitialCondition (AppCtx *user, Vec X)
 	    }
 
       /* Restore vector */
-      DPRINTF ("Restoring array to local vector\n");
+      DPRINTF ("Restoring array to local vector\n",0);
       ierr = VecRestoreArray (localX,&x); CHKERRQ (ierr);
 
       /* Insert values into global vector */
-      DPRINTF ("Inserting local vector values into global vector\n");
+      DPRINTF ("Inserting local vector values into global vector\n",0);
       ierr = DALocalToGlobal (user->da,localX,INSERT_VALUES,X); CHKERRQ (ierr);
     }
 
@@ -458,7 +501,7 @@ int InitializeProblem (AppCtx *user, Vec *xvec)
   PetscTruth twodee;
 
   /* Initialize problem parameters */
-  DPRINTF ("Initializing user->parameters\n");
+  DPRINTF ("Initializing user->parameters\n",0);
   user->mx = 20;
   user->my = 20; 
   user->mz = 20; 
@@ -494,11 +537,16 @@ int InitializeProblem (AppCtx *user, Vec *xvec)
   CHKERRQ (ierr);
   ierr = PetscOptionsHasName (PETSC_NULL, "-random", &user->random);
   CHKERRQ (ierr);
+  ierr = PetscOptionsHasName (PETSC_NULL, "-save_data", &user->save_data);
+  CHKERRQ (ierr);
+  user->load_data = -1;
+  ierr = PetscOptionsGetInt (PETSC_NULL, "-load_data", &user->load_data,
+			     PETSC_NULL); CHKERRQ (ierr);
 
   /* Create distributed array (DA) to manage parallel grid and vectors
      for principal unknowns (x) and governing residuals (f)
      Note the stencil width of 2 for this 4th-order equation. */
-  DPRINTF ("Creating distributed array\n");
+  DPRINTF ("Creating distributed array\n",0);
   Nx = PETSC_DECIDE;
   Ny = PETSC_DECIDE;
   Nz = PETSC_DECIDE;
@@ -507,7 +555,7 @@ int InitializeProblem (AppCtx *user, Vec *xvec)
   ierr = PetscOptionsGetInt (PETSC_NULL, "-Nz", &Nz, PETSC_NULL);CHKERRQ(ierr);
   if (user->threedee)
     {
-      DPRINTF ("Three Dee!\n");
+      DPRINTF ("Three Dee!\n",0);
       user->period = DA_XYZPERIODIC;
       ierr = DACreate3d (PETSC_COMM_WORLD, user->period, DA_STENCIL_BOX,
 			 user->mx, user->my, user->mz, Nx, Ny, Nz, user->mc, 2,
@@ -524,27 +572,28 @@ int InitializeProblem (AppCtx *user, Vec *xvec)
     }
 
   /* Extract global vector from DA */
-  DPRINTF ("Extracting global vector from DA...\n");
+  DPRINTF ("Extracting global vector from DA...\n",0);
   ierr = DACreateGlobalVector(user->da,&xv); CHKERRQ (ierr);
 
   /* Label PDE components */
-  DPRINTF ("Labeling PDE components\n");
+  DPRINTF ("Labeling PDE components\n",0);
   ierr = PetscMalloc (user->mc * sizeof(char*), &(user->label));CHKERRQ (ierr);
   user->label[0] = "Concentration (C)";
   /* user->label[1] = "Velocity (V)";
      user->label[2] = "Omega";
      user->label[3] = "Temperature"; */
+  ierr = DASetFieldName (user->da, 0, user->label[0]); CHKERRQ(ierr);
 
   user->x_old = 0;
 
   /* Get local vector */
-  DPRINTF ("Getting local vector\n");
+  DPRINTF ("Getting local vector\n",0);
   ierr = DACreateLocalVector (user->da, &user->localX); CHKERRQ (ierr);
 
   /* Print grid info */
   if (user->print_grid)
     {
-      DPRINTF ("Printing grid information\n");
+      DPRINTF ("Printing grid information\n",0);
       ierr = DAView(user->da,PETSC_VIEWER_STDOUT_SELF); CHKERRQ (ierr);
       ierr = DAGetCorners(user->da,&xs,&ys,&zs,&xm,&ym,&zm); CHKERRQ (ierr);
       if (!user->threedee) {
@@ -565,7 +614,7 @@ int InitializeProblem (AppCtx *user, Vec *xvec)
   }  
 
   /* Compute initial condition */
-  DPRINTF ("Forming inital condition\n");
+  DPRINTF ("Forming inital condition\n",0);
   ierr = FormInitialCondition (user, xv); CHKERRQ (ierr);
 
   *xvec = xv;
